@@ -5,35 +5,44 @@ import numpy as np
 import scipy.sparse as sp
 
 from autogoal import kb
-from autogoal.contrib.sklearn._utils import is_matrix_continuous_dense,\
-                                            is_matrix_continuous_sparse,\
-                                            is_categorical,\
-                                            is_continuous,\
-                                            is_string_list
+from autogoal.contrib.sklearn._utils import (
+    is_matrix_continuous_dense,
+    is_matrix_continuous_sparse,
+    is_categorical,
+    is_continuous,
+    is_string_list,
+)
 
 DATA_TYPE_EXAMPLES = {
-    kb.Postag():("lorem", "ipsum"), # (str, str) Tagged token
-    kb.List(kb.Postag()):[("lorem", "ipsum")] * 10, # [(str, str), (str, str)] List of tagged tokens
-    kb.List(kb.List(kb.Postag())):[[("lorem", "ipsum")] * 2], # [[(str, str), (str, str)], [(str, str), (str, str)]] List of Tagged Sentences
-    kb.Chunktag():(("lorem", "ipsum"),"ipsum"), # ((str, str), str) IOB Tagged token
-    kb.List(kb.Chunktag()):[(("lorem", "ipsum"),"ipsum")] * 10, # [((str, str), str), ((str, str), str)] List of IOB Tagged token
-    kb.List(kb.List(kb.Chunktag())):[[(("lorem", "ipsum"),"ipsum")] * 2], # [[((str, str), str), ((str, str), str)], [((str, str), str), ((str, str), str)]] List of IOB Tagged Sentences
-    kb.Stem():"ips",
-    kb.Word():"ipsum",
-    kb.Sentence():"It is the best of all movies.",
-    kb.Document():"It is the best of all movies. I actually love that action scene.",
-    kb.MatrixContinuousDense(): np.random.rand(10, 10),
-    kb.MatrixContinuousSparse(): sp.rand(10, 10),
-    kb.CategoricalVector(): np.asarray(["A"] * 5 + ["B"] * 5),
-    kb.ContinuousVector(): np.random.rand(10),
-    kb.DiscreteVector(): np.random.randint(0, 10, (10,), dtype=int),
-    kb.List(kb.Word()):["ipsu", "lorem"],
-    kb.List(kb.Document()): ["abc ipsu lorem say hello", "ipsum lorem", "abc"] * 2,
-    kb.List(kb.List(kb.Stem())): [["abc", "ipsu", "lorem"] * 10],
-    kb.List(kb.List(kb.Word())): [["abc", "ipsu", "lorem"] * 10],
-    kb.List(kb.List(kb.Sentence())): [["abc a sentence lorem"], ["ipsum lorem"], ["abc"]]
+    kb.Postag: ("lorem", "ipsum"),  # (str, str) Tagged token
+    kb.Seq[kb.Postag]: [("lorem", "ipsum")]
+    * 10,  # [(str, str), (str, str)] List of tagged tokens
+    kb.Seq[kb.Seq[kb.Postag]]: [
+        [("lorem", "ipsum")] * 2
+    ],  # [[(str, str), (str, str)], [(str, str), (str, str)]] List of Tagged Sentences
+    kb.Chunktag: (("lorem", "ipsum"), "ipsum"),  # ((str, str), str) IOB Tagged token
+    kb.Seq[kb.Chunktag]: [(("lorem", "ipsum"), "ipsum")]
+    * 10,  # [((str, str), str), ((str, str), str)] List of IOB Tagged token
+    kb.Seq[kb.Seq[kb.Chunktag]]: [
+        [(("lorem", "ipsum"), "ipsum")] * 2
+    ],  # [[((str, str), str), ((str, str), str)], [((str, str), str), ((str, str), str)]] List of IOB Tagged Sentences
+    kb.Stem: "ips",
+    kb.Word: "ipsum",
+    kb.Sentence: "It is the best of all movies.",
+    kb.Document: "It is the best of all movies. I actually love that action scene.",
+    kb.MatrixContinuousDense: np.random.rand(10, 10),
+    kb.MatrixContinuousSparse: sp.rand(10, 10),
+    kb.VectorCategorical: np.asarray(["A"] * 5 + ["B"] * 5),
+    kb.VectorContinuous: np.random.rand(10),
+    kb.VectorDiscrete: np.random.randint(0, 10, (10,), dtype=int),
+    kb.Seq[kb.Word]: ["ipsu", "lorem"],
+    kb.Seq[kb.Document]: ["abc ipsu lorem say hello", "ipsum lorem", "abc"] * 2,
+    kb.Seq[kb.Seq[kb.Stem]]: [["abc", "ipsu", "lorem"] * 10],
+    kb.Seq[kb.Seq[kb.Word]]: [["abc", "ipsu", "lorem"] * 10],
+    kb.Seq[kb.Seq[kb.Sentence]]: [["abc a sentence lorem"], ["ipsum lorem"], ["abc"]],
 }
 
+
 def is_algorithm(cls, verbose=False):
     if _is_classifier(cls):
         return "classifier"
@@ -58,81 +67,99 @@ def is_algorithm(cls, verbose=False):
 
     if _is_doc_embbeder(cls):
         return "doc_embbeder"
-    
+
     if is_pretrained_tagger(cls)[0]:
         return "trained_tagger"
-    
+
     if _is_tagger(cls):
         return "tagger"
-    
+
     return False
 
-def _is_algorithm(cls, verbose = False):
-    return  _is_stemmer(cls, verbose) or\
-            _is_lemmatizer(cls, verbose) or\
-            _is_word_tokenizer(cls, verbose) or\
-            _is_sent_tokenizer(cls, verbose) or\
-            _is_clusterer(cls, verbose) or\
-            _is_classifier(cls, verbose) or\
-            _is_word_embbeder(cls, verbose) or\
-            _is_doc_embbeder(cls, verbose) or\
-            _is_tagger(cls, verbose)
+
+def _is_algorithm(cls, verbose=False):
+    return (
+        _is_stemmer(cls, verbose)
+        or _is_lemmatizer(cls, verbose)
+        or _is_word_tokenizer(cls, verbose)
+        or _is_sent_tokenizer(cls, verbose)
+        or _is_clusterer(cls, verbose)
+        or _is_classifier(cls, verbose)
+        or _is_word_embbeder(cls, verbose)
+        or _is_doc_embbeder(cls, verbose)
+        or _is_tagger(cls, verbose)
+    )
+
 
 def _is_stemmer(cls, verbose=False):
     if hasattr(cls, "stem"):
         return True
     return False
 
+
 def _is_lemmatizer(cls, verbose=False):
     if hasattr(cls, "lemmatize"):
         return True
     return False
 
+
 def _is_word_tokenizer(cls, verbose=False):
-    if (hasattr(cls, "tokenize") or hasattr(cls, "word_tokenize")):
+    if hasattr(cls, "tokenize") or hasattr(cls, "word_tokenize"):
         return True
     return False
 
+
 def _is_sent_tokenizer(cls, verbose=False):
     if hasattr(cls, "sent_tokenize") or (hasattr(cls, "tokenize")):
         return True
     return False
 
+
 def _is_clusterer(cls, verbose=False):
-    if (hasattr(cls, "classify") and hasattr(cls, "cluster")):
+    if hasattr(cls, "classify") and hasattr(cls, "cluster"):
         return True
     return False
 
-def _is_classifier(cls, verbose = False):
-    if (hasattr(cls, "classify") and hasattr(cls, "train")):
+
+def _is_classifier(cls, verbose=False):
+    if hasattr(cls, "classify") and hasattr(cls, "train"):
         return True
     return False
 
-def _is_word_embbeder(cls, verbose = False):
-    if (hasattr(cls, "build_vocab") and hasattr(cls, "train") and hasattr(cls, "wv")):
+
+def _is_word_embbeder(cls, verbose=False):
+    if hasattr(cls, "build_vocab") and hasattr(cls, "train") and hasattr(cls, "wv"):
         return True
     return False
 
-def _is_doc_embbeder(cls, verbose = False):
-    if (hasattr(cls, "build_vocab") and hasattr(cls, "train") and hasattr(cls, "infer_vector")):
+
+def _is_doc_embbeder(cls, verbose=False):
+    if (
+        hasattr(cls, "build_vocab")
+        and hasattr(cls, "train")
+        and hasattr(cls, "infer_vector")
+    ):
         return True
     return False
 
-def _is_tagger(cls, verbose = False):
+
+def _is_tagger(cls, verbose=False):
     if hasattr(cls, "tag"):
         return True
     return False
 
-def _is_trained_tagger(cls, verbose = False):
+
+def _is_trained_tagger(cls, verbose=False):
     if hasattr(cls, "train") and _is_tagger(cls, verbose):
         return True
     return False
 
-def _is_chunker(cls, verbose = False):
+
+def _is_chunker(cls, verbose=False):
     if hasattr(cls, "parse"):
         return True
     return False
-    
+
 
 def is_stemmer(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk stemmer.
@@ -143,7 +170,7 @@ def is_stemmer(cls, verbose=False):
     >>> from sklearn.linear_model import LogisticRegression
     >>> from nltk.stem import Cistem
     >>> is_stemmer(Cistem)
-    (True, (Word(), Stem()))
+    (True, (Word, Stem))
     >>> is_stemmer(LogisticRegression)
     (False, None)
 
@@ -152,9 +179,9 @@ def is_stemmer(cls, verbose=False):
         return False, None
 
     inputs = []
-    output = kb.Stem()
+    output = kb.Stem
 
-    for input_type in [kb.Word()]:
+    for input_type in [kb.Word]:
         try:
             X = DATA_TYPE_EXAMPLES[input_type]
 
@@ -174,6 +201,7 @@ def is_stemmer(cls, verbose=False):
     else:
         return False, None
 
+
 def is_lemmatizer(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk lemmatizer.
     If True, returns the valid (input, output) types.
@@ -183,7 +211,7 @@ def is_lemmatizer(cls, verbose=False):
     >>> from sklearn.linear_model import LogisticRegression
     >>> from nltk.stem import WordNetLemmatizer
     >>> is_lemmatizer(WordNetLemmatizer)
-    (True, (Word(), Stem()))
+    (True, (Word, Stem))
     >>> is_lemmatizer(LogisticRegression)
     (False, None)
 
@@ -192,8 +220,8 @@ def is_lemmatizer(cls, verbose=False):
         return False, None
 
     inputs = []
-    output = kb.Stem()
-    for input_type in [kb.Word()]:
+    output = kb.Stem
+    for input_type in [kb.Word]:
         try:
             X = DATA_TYPE_EXAMPLES[input_type]
 
@@ -213,6 +241,7 @@ def is_lemmatizer(cls, verbose=False):
     else:
         return False, None
 
+
 def is_word_tokenizer(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk word tokenizer algorithm.
     If True, returns the valid (input, output) types.
@@ -222,7 +251,7 @@ def is_word_tokenizer(cls, verbose=False):
     >>> from sklearn.linear_model import LogisticRegression
     >>> from nltk.tokenize import TweetTokenizer
     >>> is_word_tokenizer(TweetTokenizer)
-    (True, (Sentence(), List(Word())))
+    (True, (Sentence, List(Word)))
     >>> is_word_tokenizer(LogisticRegression)
     (False, None)
 
@@ -231,9 +260,9 @@ def is_word_tokenizer(cls, verbose=False):
         return False, None
 
     inputs = []
-    output = kb.List(kb.Word())
+    output = kb.Seq[kb.Word]
 
-    for input_type in [kb.Sentence()]:
+    for input_type in [kb.Sentence]:
         try:
             X = DATA_TYPE_EXAMPLES[input_type]
 
@@ -253,6 +282,7 @@ def is_word_tokenizer(cls, verbose=False):
     else:
         return False, None
 
+
 def is_sent_tokenizer(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk sentence tokenizer.
     If True, returns the valid (input, output) types.
@@ -262,7 +292,7 @@ def is_sent_tokenizer(cls, verbose=False):
     >>> from sklearn.linear_model import LogisticRegression
     >>> from nltk.tokenize import PunktSentenceTokenizer
     >>> is_sent_tokenizer(PunktSentenceTokenizer)
-    (True, (Document(), List(Sentence())))
+    (True, (Document, List(Sentence)))
     >>> is_sent_tokenizer(LogisticRegression)
     (False, None)
 
@@ -271,9 +301,9 @@ def is_sent_tokenizer(cls, verbose=False):
         return False, None
 
     inputs = []
-    output = kb.List(kb.Sentence())
+    output = kb.Seq(kb.Sentence)
 
-    for input_type in [kb.Document()]:
+    for input_type in [kb.Document]:
         try:
             X = DATA_TYPE_EXAMPLES[input_type]
 
@@ -293,6 +323,7 @@ def is_sent_tokenizer(cls, verbose=False):
     else:
         return False, None
 
+
 def is_clusterer(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk clusterer.
     If True, returns the valid (input, output) types.
@@ -334,6 +365,7 @@ def is_clusterer(cls, verbose=False):
     else:
         return False, None
 
+
 def is_classifier(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk classifier.
     If True, returns the valid (input, output) types.
@@ -343,7 +375,7 @@ def is_classifier(cls, verbose=False):
 
     inputs = []
 
-    #TODO: Fix somehow compatibility with nltk classifiers
+    # TODO: Fix somehow compatibility with nltk classifiers
     inputs = combine_types(*inputs)
 
     if inputs:
@@ -351,6 +383,7 @@ def is_classifier(cls, verbose=False):
     else:
         return False, None
 
+
 def is_tagger(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk pos tagger.
     If True, returns the valid (input, output) types.
@@ -360,7 +393,7 @@ def is_tagger(cls, verbose=False):
     >>> from nltk.tag import AffixTagger
     >>> from nltk.tokenize import PunktSentenceTokenizer
     >>> is_tagger(AffixTagger)
-    (True, (List(List(Word())), List(List(Postag()))))
+    (True, (List(List(Word)), List(List(Postag()))))
     >>> is_tagger(LogisticRegression)
     (False, None)
 
@@ -369,14 +402,14 @@ def is_tagger(cls, verbose=False):
         return False, None
 
     inputs = []
-    output = kb.List(kb.List(kb.Postag()))
+    output = kb.Seq(kb.Seq[kb.Postag])
 
-    for input_type in [kb.List(kb.List(kb.Word()))]:
+    for input_type in [kb.Seq(kb.Seq[kb.Word])]:
         try:
             X = DATA_TYPE_EXAMPLES[input_type]
 
-            X_train = [[(word,word) for word in sentence] for sentence in X]
-            
+            X_train = [[(word, word) for word in sentence] for sentence in X]
+
             tagger = cls(train=X_train)
             # tagger = cls()
             y = tagger.tag_sents(X)
@@ -394,13 +427,14 @@ def is_tagger(cls, verbose=False):
     else:
         is_ptt = is_pretrained_tagger(cls, verbose)
         is_ckr = is_chunker(cls, verbose)
-        
+
         if is_ptt[0]:
             return is_ptt
         if is_ckr[0]:
             return is_ckr
         return False, None
 
+
 def is_chunker(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk chunker.
     If True, returns the valid (input, output) types.
@@ -410,7 +444,7 @@ def is_chunker(cls, verbose=False):
     >>> from nltk.chunk.named_entity import NEChunkParserTagger
     >>> from nltk.tokenize import PunktSentenceTokenizer
     >>> is_chunker(NEChunkParserTagger)
-    (True, (List(List(Tuple(Word(), Word()))), List(List(Tuple(Tuple(Word(), Word()), Word())))))
+    (True, (List(List(Tuple(Word, Word))), List(List(Tuple(Tuple(Word, Word), Word)))))
     >>> is_chunker(PunktSentenceTokenizer)
     (False, None)
 
@@ -419,14 +453,17 @@ def is_chunker(cls, verbose=False):
         return False, None
 
     inputs = []
-    output = kb.List(kb.List(kb.Chunktag()))
+    output = kb.Seq(kb.Seq[kb.Chunktag])
 
-    for input_type in [kb.List(kb.List(kb.Postag()))]:
+    for input_type in [kb.Seq(kb.Seq[kb.Postag])]:
         try:
             X = DATA_TYPE_EXAMPLES[input_type]
 
-            X_train = [[((word, postag), postag) for word, postag in sentence] for sentence in X]
-            
+            X_train = [
+                [((word, postag), postag) for word, postag in sentence]
+                for sentence in X
+            ]
+
             chunker = cls(train=X_train)
             y = chunker.tag_sents(X)
 
@@ -443,6 +480,7 @@ def is_chunker(cls, verbose=False):
     else:
         return False, None
 
+
 def is_pretrained_tagger(cls, verbose=False):
     """Determine if `cls` corresponds to something that resembles an nltk sentence tokenizer.
     If True, returns the valid (input, output) types.
@@ -452,7 +490,7 @@ def is_pretrained_tagger(cls, verbose=False):
     >>> from nltk.tag import AffixTagger
     >>> from nltk.tag.perceptron import PerceptronTagger
     >>> is_pretrained_tagger(PerceptronTagger)
-    (True, (List(Word()), List(Tuple(Word(), Word()))))
+    (True, (List(Word), List(Tuple(Word, Word))))
     >>> is_pretrained_tagger(AffixTagger)
     (False, None)
 
@@ -461,14 +499,14 @@ def is_pretrained_tagger(cls, verbose=False):
         return False, None
 
     inputs = []
-    output = kb.List(kb.Postag())
+    output = kb.Seq[kb.Postag]
 
-    for input_type in [kb.List(kb.Word())]:
+    for input_type in [kb.Seq[kb.Word]]:
         try:
             X = DATA_TYPE_EXAMPLES[input_type]
-            
+
             tagger = cls()
-            
+
             y = tagger.tag(X)
 
             assert DATA_RESOLVERS[output](y)
@@ -483,10 +521,12 @@ def is_pretrained_tagger(cls, verbose=False):
         return True, (inputs, output)
     else:
         return False, None
-    
+
+
 def is_data_type(X, data_type):
     return DATA_RESOLVERS[data_type](X)
 
+
 IO_TYPE_HANDLER = [
     is_stemmer,
     is_lemmatizer,
@@ -499,6 +539,7 @@ IO_TYPE_HANDLER = [
     # is_doc_embbeder
 ]
 
+
 def get_input_output(cls, verbose=False):
     for func in IO_TYPE_HANDLER:
         matches, types = func(cls, verbose=verbose)
@@ -507,6 +548,7 @@ def get_input_output(cls, verbose=False):
 
     return None, None
 
+
 def combine_types(*types):
     if len(types) == 1:
         return types[0]
@@ -518,6 +560,7 @@ def combine_types(*types):
 
     return None
 
+
 def is_word_list(obj):
     """Determines if `obj` is a sequence of sequence of strings.
 
@@ -539,6 +582,7 @@ def is_word_list(obj):
     except:
         return False
 
+
 def is_word_list_list(obj):
     """Determines if `obj` is a sequence of sequence of strings.
 
@@ -561,6 +605,7 @@ def is_word_list_list(obj):
     except:
         return False
 
+
 def is_word(obj):
     """Determines if `obj` is a sequence of sequence of strings.
 
@@ -577,6 +622,7 @@ def is_word(obj):
     except:
         return False
 
+
 def is_sentence(obj):
     """Determines if `obj` is a sentence strings.
 
@@ -593,12 +639,14 @@ def is_sentence(obj):
     except:
         return False
 
+
 def is_sentence_list(obj):
     try:
         return all([is_sentence(x) for x in obj])
     except:
         return False
 
+
 def is_text_list_list(obj):
     """Determines if `obj` is a sequence of sequence of strings.
 
@@ -621,6 +669,7 @@ def is_text_list_list(obj):
     except:
         return False
 
+
 def is_tag(obj):
     """Determines if `obj` is a tuple of two strings.
 
@@ -633,10 +682,15 @@ def is_tag(obj):
 
     """
     try:
-        return isinstance(obj, tuple) and len(obj) == 2 and all((isinstance(x, str) or x == None) for x in obj)
+        return (
+            isinstance(obj, tuple)
+            and len(obj) == 2
+            and all((isinstance(x, str) or x == None) for x in obj)
+        )
     except:
         return False
-    
+
+
 def is_tag_list(obj):
     """Determines if `obj` is a list of tuple of two strings.
 
@@ -653,6 +707,7 @@ def is_tag_list(obj):
     except:
         return False
 
+
 def is_tagged_sentence_list(obj):
     """Determines if `obj` is a list(list(tuple(str, str)))
 
@@ -682,10 +737,16 @@ def is_chunk(obj):
 
     """
     try:
-        return isinstance(obj, tuple) and len(obj) == 2 and is_tag(obj[0]) and isinstance(obj[1], str)
+        return (
+            isinstance(obj, tuple)
+            and len(obj) == 2
+            and is_tag(obj[0])
+            and isinstance(obj[1], str)
+        )
     except:
         return False
 
+
 def is_chunk_list(obj):
     """Determines if `obj` is a list(tuple(tuple(str, str), str)).
 
@@ -702,6 +763,7 @@ def is_chunk_list(obj):
     except:
         return False
 
+
 def is_chunked_sentence_list(obj):
     """Determines if `obj` is a list(list(tuple(tuple(str, str), str))).
 
@@ -720,22 +782,22 @@ def is_chunked_sentence_list(obj):
 
 
 DATA_RESOLVERS = {
-    kb.Postag():is_tag,
-    kb.List(kb.Postag()):is_tag_list,
-    kb.List(kb.List(kb.Postag())):is_tagged_sentence_list,
-    kb.Chunktag:is_chunk,
-    kb.List(kb.Chunktag()):is_chunk_list,
-    kb.List(kb.List(kb.Chunktag())):is_chunked_sentence_list,
-    kb.Stem():is_word,
-    kb.Word():is_word,
-    kb.Sentence():is_sentence,
-    kb.Document():is_sentence,
-    kb.MatrixContinuousDense(): is_matrix_continuous_dense,
-    kb.MatrixContinuousSparse(): is_matrix_continuous_sparse,
-    kb.CategoricalVector(): is_categorical,
-    kb.ContinuousVector(): is_continuous,
-    kb.List(kb.Word()): is_word_list,
-    kb.List(kb.Sentence()): is_sentence_list,
-    kb.List(kb.List(kb.Stem())): is_word_list_list,
-    kb.List(kb.List(kb.Word())): is_word_list_list,
-}
\ No newline at end of file
+    kb.Postag: is_tag,
+    kb.Seq[kb.Postag]: is_tag_list,
+    kb.Seq[kb.Seq[kb.Postag]]: is_tagged_sentence_list,
+    kb.Chunktag: is_chunk,
+    kb.Seq[kb.Chunktag]: is_chunk_list,
+    kb.Seq[kb.Seq[kb.Chunktag]]: is_chunked_sentence_list,
+    kb.Stem: is_word,
+    kb.Word: is_word,
+    kb.Sentence: is_sentence,
+    kb.Document: is_sentence,
+    kb.MatrixContinuousDense: is_matrix_continuous_dense,
+    kb.MatrixContinuousSparse: is_matrix_continuous_sparse,
+    kb.VectorCategorical: is_categorical,
+    kb.VectorContinuous: is_continuous,
+    kb.Seq[kb.Word]: is_word_list,
+    kb.Seq[kb.Sentence]: is_sentence_list,
+    kb.Seq[kb.Seq[kb.Stem]]: is_word_list_list,
+    kb.Seq[kb.Seq[kb.Word]]: is_word_list_list,
+}
