@@ -30,5 +30,16 @@ except:
     raise ImportError()
 
 
+# Filter out sklearn convergence and joblib warnings
+import warnings
+from sklearn.exceptions import ConvergenceWarning
+
+warnings.filterwarnings("ignore", message=".*resource_tracker.*")
+warnings.filterwarnings("ignore", category=ConvergenceWarning)
+warnings.filterwarnings("ignore", module="sklearn")
+warnings.filterwarnings("ignore", module="scipy")
+warnings.filterwarnings("ignore", module="numpy")
+
+
 from ._generated import *
 from ._manual import *
