@@ -5,14 +5,14 @@ This module generates a random dataset useful for quickly testing the interface
 import numpy as np
 
 
-def load(samples=100, classes=2, features=10, seed=None):
+def generate(samples=100, classes=2, features=10, exponent=1, error=0.1, seed=None):
     """
     Create a random X,y pair.
 
     ##### Examples
 
     ```python
-    >>> X, y = load(samples=4, features=2, seed=0)
+    >>> X, y = generate(samples=4, features=2, seed=0)
     >>> print(X)
     [[0.5488135  0.71518937]
      [0.60276338 0.54488318]
@@ -30,4 +30,4 @@ def load(samples=100, classes=2, features=10, seed=None):
     X = np.random.random((samples, features))
     y = np.random.randint(0, classes, samples).astype(str)
 
-    return X, y
\ No newline at end of file
+    return X, y
