@@ -1,24 +1,34 @@
 import random
 
+from autogoal.datasets import download, datapath
+
 
 def load(max_examples=None):
-    from nltk.corpus import movie_reviews
+    try:
+        download("movie_reviews")
+    except:
+        print(
+            "Error loading data. This may be caused due to bad connection. Please delete badly downloaded data and retry"
+        )
+        raise
 
     sentences = []
     classes = []
 
-    ids = list(movie_reviews.fileids())
+    path = datapath("movie_reviews")
+
+    ids = list(path.rglob("*.txt"))
     random.shuffle(ids)
 
     for fd in ids:
-        if fd.startswith("neg/"):
+        if "neg/" in str(fd):
             cls = "neg"
         else:
             cls = "pos"
 
-        fp = movie_reviews.open(fd)
-        sentences.append(fp.read())
-        classes.append(cls)
+        with fd.open() as fp:
+            sentences.append(fp.read())
+            classes.append(cls)
 
         if max_examples and len(classes) >= max_examples:
             break
@@ -34,10 +44,8 @@ def make_fn(test_size=0.25, examples=None):
     X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=test_size)
 
     def fitness_fn(pipeline):
-        pipeline.send("train")
-        pipeline.run((X_train, y_train))
-        pipeline.send("eval")
-        _, y_pred = pipeline.run((X_test, [None] * len(y_test)))
+        pipeline.fit(X_train, y_train)
+        y_pred = pipeline.predict(X_test)
 
         return accuracy_score(y_test, y_pred)
 
