@@ -2,10 +2,10 @@ from ._base import Grammar, Sampler
 from ._cfg import (
     generate_cfg,
     ContextFreeGrammar,
-    Discrete,
-    Continuous,
-    Categorical,
-    Boolean,
+    DiscreteValue,
+    ContinuousValue,
+    CategoricalValue,
+    BooleanValue,
     Union,
     Symbol,
     CfgInitializer,
