@@ -392,6 +392,9 @@ class ReplaySampler:
         return self._run("categorical", *args, **kwargs)
 
     def __getattr__(self, attr):
+        if attr == "sampler":
+            return self.__dict__.get("sampler")
+
         return getattr(self.sampler, attr)
 
 
@@ -474,7 +477,9 @@ class MeanDevParam(ModelParam):
         average = np.average(values, weights=weights)
         variance = np.average((values - average) ** 2, weights=weights)
 
-        return MeanDevParam(average, math.sqrt(variance), initial_params=self.initial_params)
+        return MeanDevParam(
+            average, math.sqrt(variance), initial_params=self.initial_params
+        )
 
 
 @nice_repr
@@ -590,3 +595,71 @@ def merge_updates(*updates: Sequence[Dict]) -> Dict:
             result[key].extend(value)
 
     return result
+
+
+class ExhaustiveSampler:
+    """
+    Performs an exhaustive sampling of the parameter space.
+    """
+
+    # The way this sampler works is by building an explicit representation of
+    # a parameter space in the form a search tree. As you sample values it will
+    # create nodes that represent each decision and the possible outcomes.
+
+    # We'll build a tree that represents all possible combinations in the parameter space.
+    # Each node in the tree contains set of values, and the children correspond to the nodes
+    # that represent the subsequent options for each value.
+
+    # For example, if our parameter space has two variables, `X=[1,2,3]` and `Y=['a', 'b']`,
+    # we'll represent it in a tree that looks like this:
+    #
+    # ```
+    #  [ 1   ,   2   ,   3 ]
+    #    |       |       |
+    #  [a,b]   [a,b]   [a,b]
+    #  /   \   /   \   /   \
+    # 1,a 1,b 2,a 2,b 3,a 3,b
+    # ```
+
+    # In each node we'll store a handle and a set of values for the distribution.
+
+    # The challenge here is build the paramter tree at the same time that we are sampling.
+    # Since we don't know beforehand what of the structure of the parameter space,
+    # we have to discover it as the different sampling methods are being invoked.
+
+    # However, since sampling is a never-ending process, we will have to explicitely
+    # tell this sampler when we finished with one instance. Otherwise we can't know when
+    # did we reach a leaf.
+
+    def __init__(self) -> None:
+        self._root = ExhaustiveSampler.Node()
+        self._current_node = self._root
+
+    def _sample_next(self, distribution, params, handle):
+        # At any moment when we call a sampling method, we are at some node in the parameter space.
+        # In the simplest case that node is not initialized, i.e., we have never sampled from it
+        # before.
+
+        if not self._current_node.is_initialized():
+            self._current_node.initialize(distribution, params, handle)
+
+        # Then we'll sample from that node, which in this case will only return the next
+        # value. At the same time, we'll recurse down the corresponding children.
+
+        self._current_node, value = self._current_node.sample()
+        return value
+
+    class Node:
+        def is_initialized(self) -> bool:
+            return hasattr(self, "handle")
+
+        def initialize(self, distribution, params, handle) -> None:
+            self.handle = handle
+            self.values = getattr(self, f"initialize_{distribution}")(**params)
+
+            # In this dictionary we will store as values the nodes that represents the distributions
+            # that will be invoked after returning the corresponding result stored at each key.
+            self.children = {}
+
+        def sample(self):
+            pass
